@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py,v 1.49 2006/06/21 15:05:42 richb Exp $"
-__version__   = "$Revision: 1.49 $"
-__date__      = "$Date: 2006/06/21 15:05:42 $"
+__id__        = "$Id: braillegenerator.py,v 1.54 2006/08/01 15:26:57 wwalker Exp $"
+__version__   = "$Revision: 1.54 $"
+__date__      = "$Date: 2006/08/01 15:26:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -34,7 +34,7 @@ import math
 import atspi
 import braille
 import debug
-import orca
+import orca_state
 import rolenames
 import settings
 import util
@@ -405,7 +405,7 @@ class BrailleGenerator:
         else:
             text = util.appendString(text, "< >")
 
-        if obj == orca.locusOfFocus:
+        if obj == orca_state.locusOfFocus:
             text = util.appendString(text, self._getTextForRole(obj))
             text = util.appendString(text, self._getTextForAvailability(obj))
             text = util.appendString(text,
@@ -645,7 +645,7 @@ class BrailleGenerator:
         text = util.appendString(text, util.getDisplayedText(obj))
         text = util.appendString(text, rolenames.getBrailleForRoleName(obj))
 
-        if obj == orca.locusOfFocus:
+        if obj == orca_state.locusOfFocus:
             text = util.appendString(text, self._getTextForAvailability(obj))
             text = util.appendString(text,
                                       self._getTextForAccelerator(obj),
@@ -687,7 +687,7 @@ class BrailleGenerator:
         text = util.appendString(text, util.getDisplayedLabel(obj))
         text = util.appendString(text, util.getDisplayedText(obj))
 
-        if obj == orca.locusOfFocus:
+        if obj == orca_state.locusOfFocus:
             text = util.appendString(text, self._getTextForAvailability(obj))
             text = util.appendString(text,
                                       self._getTextForAccelerator(obj),
@@ -753,7 +753,7 @@ class BrailleGenerator:
         text = util.appendString(text, util.getDisplayedLabel(obj))
         text = util.appendString(text, util.getDisplayedText(obj))
 
-        if obj == orca.locusOfFocus:
+        if obj == orca_state.locusOfFocus:
             text = util.appendString(text, self._getTextForAvailability(obj))
             text = util.appendString(text,
                                       self._getTextForAccelerator(obj),
@@ -811,6 +811,14 @@ class BrailleGenerator:
         text = ""
         text = util.appendString(text, util.getDisplayedLabel(obj))
         text = util.appendString(text, util.getDisplayedText(obj))
+
+        # In Java, some push buttons don't have label and text.
+        # In this case, we'll add to presentation the object description,
+        # if exists.
+        #
+        if (not text) and (obj.description):
+            text = util.appendString(text, obj.description)
+
         text = util.appendString(text, self._getTextForRole(obj))
 
         regions = []
@@ -836,6 +844,13 @@ class BrailleGenerator:
         text = util.appendString(text, util.getDisplayedLabel(obj))
         text = util.appendString(text, util.getDisplayedText(obj))
 
+        # In Java, some toggle buttons don't have label and text.
+        # In this case, we'll add to presentation the object description,
+        # if exists.
+        #
+        if (not text) and (obj.description):
+            text = util.appendString(text, obj.description)
+
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
             text = util.appendString(text, "&=y")
         else:
@@ -871,7 +886,8 @@ class BrailleGenerator:
         else:
             text = util.appendString(text, "& y")
 
-        if obj == orca.locusOfFocus:
+        if obj == orca_state.locusOfFocus:
+            text = util.appendString(text, self._getTextForRole(obj))
             text = util.appendString(text, self._getTextForAvailability(obj))
             text = util.appendString(text,
                                       self._getTextForAccelerator(obj),
@@ -1039,7 +1055,7 @@ class BrailleGenerator:
                            ((row == 0 or row == parent.table.nRows-1) and \
                             parent.lastColumn == column)
 
-                if speakAll == True:
+                if speakAll:
                     focusRowRegion = None
                     for i in range(0, parent.table.nColumns):
                         accRow = parent.table.getAccessibleAt(row, i)
@@ -1350,6 +1366,21 @@ class BrailleGenerator:
                     result = self.getBrailleRegions(parent, False)
                     regions.extend(result[0])
 
+            # [[[TODO: HACK - we've discovered oddness in hierarchies
+            # such as the gedit Edit->Preferences dialog.  In this
+            # dialog, we have labeled groupings of objects.  The
+            # grouping is done via a FILLER with two children - one
+            # child is the overall label, and the other is the
+            # container for the grouped objects.  When we detect this,
+            # we add the label to the overall context.]]]
+            #
+            if parent.role == rolenames.ROLE_FILLER:
+                label = util.getDisplayedLabel(parent)
+                if label and len(label) and not label.isspace():
+                    regions.append(braille.Region(" "))
+                    result = self.getBrailleRegions(parent, False)
+                    regions.extend(result[0])
+
             parent = parent.parent
 
         regions.reverse()
