@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -17,6 +17,14 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+"""Provides the default implementation for flat review for Orca."""
+
+__id__        = "$Id: flat_review.py,v 1.40 2006/06/30 21:04:44 richb Exp $"
+__version__   = "$Revision: 1.40 $"
+__date__      = "$Date: 2006/06/30 21:04:44 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 import re
 import sys
 
@@ -1058,6 +1066,9 @@ def getZonesFromAccessible(accessible, cliprect):
     - cliprect: the extents that the Zones must fit inside.
     """
 
+    if not accessible.component:
+        return []
+
     # Get the component extents in screen coordinates.
     #
     extents = accessible.component.getExtents(0)
@@ -1087,6 +1098,16 @@ def getZonesFromAccessible(accessible, cliprect):
                 offset,
                 atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
 
+            # [[[WDW - HACK: well...gnome-terminal sometimes wants to
+            # give us outrageous values back from getTextAtOffset
+            # (see http://bugzilla.gnome.org/show_bug.cgi?id=343133),
+            # so we try to handle it.]]]
+            #
+            if startOffset < 0:
+                break
+            if endOffset < offset:
+                break
+
             # [[[WDW - HACK: this is here because getTextAtOffset
             # tends not to be implemented consistently across toolkits.
             # Sometimes it behaves properly (i.e., giving us an endOffset
@@ -1223,7 +1244,7 @@ def getZonesFromAccessible(accessible, cliprect):
             string = accessible.description
         else:
             string = ""
-            
+
         if string == "":
             # [[[TODO: WDW - ooohhhh....this is going to be a headache.
             # We want to synthesize a string for objects that are there,
