@@ -17,10 +17,14 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""Manages the magnifier for orca.  [[[TODO: WDW - this is very very early
-in development.  One might even say it is pre-prototype.  Magnification
-has also been disabled for now - logged as bugzilla bug 319643.]]]
-"""
+"""Manages the magnifier for orca.  [[[TODO: WDW - this is very very
+early in development.  One might even say it is pre-prototype.]]]"""
+
+__id__        = "$Id: mag.py,v 1.29 2006/07/28 17:33:16 wwalker Exp $"
+__version__   = "$Revision: 1.29 $"
+__date__      = "$Date: 2006/07/28 17:33:16 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import bonobo
 try:
@@ -93,6 +97,11 @@ _zoomer = None
 #
 _lastMouseEventTime = time.time()
 
+# If True, we're using gnome-mag >= 0.13.1 that allows us to control
+# where to draw the cursor and crosswires.
+#
+_pollMouseDisabled = False
+
 def __setROI(rect):
     """Sets the region of interest.
 
@@ -162,6 +171,11 @@ def __setROIPush(x, y):
         needNewROI = True
         newROI.y2 = y
         newROI.y1 = y - _roiHeight
+
+    # Well...we'll always update the ROI so the new gnome-mag API
+    # will redraw the crosswires for us.
+    #
+    #if needNewROI:
     if True:
         __setROI(newROI)
 
@@ -206,6 +220,12 @@ def __onMouseEvent(e):
 
     [x, y] = [e.detail1, e.detail2]
 
+    # If True, we're using gnome-mag >= 0.13.1 that allows us to
+    # control where to draw the cursor and crosswires.
+    #
+    if _pollMouseDisabled:
+        _zoomer.setPointerPos(x, y)
+
     if settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PUSH:
         __setROIPush(x, y)
     elif settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PROPORTIONAL:
@@ -254,6 +274,7 @@ def applySettings():
     global _minROIY
     global _maxROIX
     global _maxROIY
+    global _pollMouseDisabled
 
     ########################################################################
     #                                                                      #
@@ -361,6 +382,15 @@ def applySettings():
     zoomerPBag = _zoomer.getProperties()
     bonobo.pbclient_set_boolean(zoomerPBag, "is-managed", True)
 
+    # Try to use gnome-mag >= 0.13.1 to allow us to control where to
+    # draw the cursor and crosswires.
+    #
+    try:
+        bonobo.pbclient_set_boolean(zoomerPBag, "poll-mouse", False)
+        _pollMouseDisabled = True
+    except:
+        _pollMouseDisabled = False
+        
     _zoomer.setMagFactor(settings.magZoomFactor, settings.magZoomFactor)
 
     bonobo.pbclient_set_boolean(
@@ -440,7 +470,6 @@ def magnifyAccessible(event, obj):
 
     if visibleX and visibleY:
         _zoomer.markDirty(_roi)
-        return
 
     # The algorithm is devised to move the ROI as little as possible, yet
     # favor the top left side of the object [[[TODO: WDW - the left/right
