@@ -17,6 +17,14 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+"""The main module for the Orca screen reader."""
+
+__id__        = "$Id: orca.py,v 1.150 2006/07/28 17:57:16 richb Exp $"
+__version__   = "$Revision: 1.150 $"
+__date__      = "$Date: 2006/07/28 17:57:16 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -37,17 +45,22 @@ import time
 import atspi
 import braille
 import debug
+import httpserver
 import keynames
 import keybindings
 import mag
+import orca_state
 import platform
 import rolenames
 import settings
 import speech
+    
 import threading
+import util
 
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
+from input_event import MouseButtonEvent
 from input_event import InputEventHandler
 
 from orca_i18n import _           # for gettext support
@@ -56,19 +69,6 @@ from orca_i18n import _           # for gettext support
 #
 _userSettings = None
 
-# set each time a keyboard or braille event is received.
-#
-
-# The InputEvent instance representing the last input event.  This is
-# set each time a keyboard or braille event is received.
-#
-lastInputEvent = None
-
-# A new modifier to use (set by the press of any key in the
-# settings.orcaModifierKeys list) to represent the Orca modifier.
-#
-MODIFIER_ORCA = 8
-
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING PRESENTATION MANAGERS                           #
@@ -129,10 +129,6 @@ def _switchToNextPresentationManager(script=None, inputEvent=None):
 #                                                                      #
 ########################################################################
 
-# The Accessible that has visual focus.
-#
-locusOfFocus = None
-
 def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     """Sets the locus of focus (i.e., the object with visual focus) and
     notifies the current presentation manager of the change.
@@ -143,29 +139,29 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     - notifyPresentationManager: if True, propagate this event
     """
 
-    global locusOfFocus
-
-    if obj == locusOfFocus:
+    if obj == orca_state.locusOfFocus:
         return
 
-    oldLocusOfFocus = locusOfFocus
+    oldLocusOfFocus = orca_state.locusOfFocus
     if oldLocusOfFocus and not oldLocusOfFocus.valid:
         oldLocusOfFocus = None
 
-    locusOfFocus = obj
-    if locusOfFocus and not locusOfFocus.valid:
-        locusOfFocus = None
+    orca_state.locusOfFocus = obj
+    if orca_state.locusOfFocus and not orca_state.locusOfFocus.valid:
+        orca_state.locusOfFocus = None
 
-    if locusOfFocus:
+    if orca_state.locusOfFocus:
         appname = ""
-        if not locusOfFocus.app:
+        if not orca_state.locusOfFocus.app:
             appname = "None"
         else:
-            appname = "'" + locusOfFocus.app.name + "'"
+            appname = "'" + orca_state.locusOfFocus.app.name + "'"
 
         debug.println(debug.LEVEL_FINE,
                       "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
-                      % (appname, locusOfFocus.name, locusOfFocus.role))
+                      % (appname,
+                         orca_state.locusOfFocus.name,
+                         orca_state.locusOfFocus.role))
 
         if event:
             debug.println(debug.LEVEL_FINE,
@@ -183,7 +179,9 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
 
     if notifyPresentationManager and _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].\
-            locusOfFocusChanged(event, oldLocusOfFocus, locusOfFocus)
+            locusOfFocusChanged(event,
+                                oldLocusOfFocus,
+                                orca_state.locusOfFocus)
 
 def visualAppearanceChanged(event, obj):
     """Called (typically by scripts) when the visual appearance of an object
@@ -203,38 +201,6 @@ def visualAppearanceChanged(event, obj):
         _PRESENTATION_MANAGERS[_currentPresentationManager].\
             visualAppearanceChanged(event, obj)
 
-def isInActiveApp(obj):
-    """Returns True if the given object is from the same application that
-    currently has keyboard focus.
-
-    Arguments:
-    - obj: an Accessible object
-    """
-
-    if not obj:
-        return False
-    else:
-        return locusOfFocus and (locusOfFocus.app == obj.app)
-
-def findActiveWindow():
-    """Traverses the list of known apps looking for one who has an
-    immediate child (i.e., a window) whose state includes the active state.
-
-    Returns the Python Accessible of the window that's active or None if
-    no windows are active.
-    """
-
-    window = None
-    apps = atspi.getKnownApplications()
-    for app in apps:
-        for i in range(0, app.childCount):
-            state = app.child(i).state
-            if state.count(atspi.Accessibility.STATE_ACTIVE) > 0:
-                window = app.child(i)
-                break
-
-    return window
-
 def _onChildrenChanged(e):
     """Tracks children-changed events on the desktop to determine when
     apps start and stop.
@@ -258,6 +224,24 @@ def _onChildrenChanged(e):
             shutdown()
             return
 
+def _onMouseButton(e):
+    """Tracks mouse button events, stopping any speech in progress.
+
+    Arguments:
+    - e: at-spi event from the at-api registry
+    """
+
+    event = atspi.Event(e)
+    orca_state.lastInputEvent = MouseButtonEvent(event)
+
+    # A mouse button event looks like: mouse:button:1p, where the
+    # number is the button number and the 'p' is either 'p' or 'r',
+    # meaning pressed or released.  We only want to stop speech on
+    # button presses.
+    #
+    if event.type.endswith("p"):
+        speech.stop()
+
 ########################################################################
 #                                                                      #
 # Keyboard Event Recording Support                                     #
@@ -417,92 +401,25 @@ def cycleDebugLevel(script=None, inputEvent=None):
     return True
 
 def printApps(script=None, inputEvent=None):
-    """Prints a list of all applications to stdout
-
-    Arguments:
-    - inputEvent: the InputEvent instance that caused this to be called.
-
-    Returns True indicating the event should be consumed.
-    """
-
-    level = debug.LEVEL_OFF
-
-    apps = atspi.getKnownApplications()
-    debug.println(level, "There are %d Accessible applications" % len(apps))
-    for app in apps:
-        debug.printDetails(level, "  App: ", app, False)
-        for i in range(0, app.childCount):
-            child = app.child(i)
-            debug.printDetails(level, "    Window: ", child, False)
-            if child.parent != app:
-                debug.println(level,
-                              "      WARNING: child's parent is not app!!!")
-
+    """Prints a list of all applications to stdout."""
+    util.printApps()
     return True
 
 def printActiveApp(script=None, inputEvent=None):
-    """Prints the active application.
-
-    Arguments:
-    - inputEvent: the key event (if any) which caused this to be called.
-
-    Returns True indicating the event should be consumed.
-    """
-
-    level = debug.LEVEL_OFF
-
-    window = findActiveWindow()
-    if not window:
-        debug.println(level, "Active application: None")
-    else:
-        app = window.app
-        if not app:
-            debug.println(level, "Active application: None")
-        else:
-            debug.println(level, "Active application: %s" % app.name)
-
+    """Prints the currently active application."""
+    util.printActiveApp()
     return True
 
 def printAncestry(script=None, inputEvent=None):
     """Prints the ancestry for the current locusOfFocus"""
-    atspi.printAncestry(locusOfFocus)
+    util.printAncestry(orca_state.locusOfFocus)
     return True
 
 def printHierarchy(script=None, inputEvent=None):
     """Prints the application for the current locusOfFocus"""
-    if locusOfFocus:
-        atspi.printHierarchy(locusOfFocus.app, locusOfFocus)
-    return True
-
-########################################################################
-#                                                                      #
-# METHODS FOR HANDLING LEARN MODE.                                     #
-#                                                                      #
-########################################################################
-
-def enterLearnMode(script=None, inputEvent=None):
-    """Turns learn mode on.  The user must press the escape key to exit
-    learn mode.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    speech.speak(
-        _("Entering learn mode.  Press any key to hear its function. " \
-          + "To exit learn mode, press the escape key."))
-    braille.displayMessage(_("Learn mode.  Press escape to exit."))
-    settings.learnModeEnabled = True
-    return True
-
-def exitLearnMode(script=None, inputEvent=None):
-    """Turns learn mode off.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    speech.speak(_("Exiting learn mode."))
-    braille.displayMessage(_("Exiting learn mode."))
-    settings.learnModeEnabled = False
+    if orca_state.locusOfFocus:
+        util.printHierarchy(orca_state.locusOfFocus.app,
+                            orca_state.locusOfFocus)
     return True
 
 ########################################################################
@@ -616,10 +533,6 @@ def _isActionKey(event_string):
                   "orca._echoActionKey: returning: %s" % reply)
     return reply
 
-# Records the last time a key was echoed.
-#
-lastKeyEchoTime = None
-
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -632,12 +545,11 @@ def _keyEcho(event):
     - event: an AT-SPI DeviceEvent
     """
 
-    global lastKeyEchoTime
-
     # If this keyboard event was for an object like a password text
     # field, then don't echo it.
     #
-    if locusOfFocus and locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT:
+    if orca_state.locusOfFocus \
+        and (orca_state.locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT):
         return
 
     event_string = event.event_string
@@ -647,6 +559,8 @@ def _keyEcho(event):
     voices = settings.voices
     voice = voices[settings.DEFAULT_VOICE]
 
+    lockState = None
+
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
     # echoed.
@@ -671,9 +585,10 @@ def _keyEcho(event):
             modifiers = event.modifiers
             if event_string == _("Caps_Lock"):
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    event_string += " Off"
+                    lockState = _(" off")
                 else:
-                    event_string += " On"
+                    lockState = _(" on")
+
             elif event_string == _("Num_Lock"):
                 # [[[TODO: richb - we are not getting a correct modifier
                 # state value returned when Num Lock is turned off.
@@ -681,9 +596,9 @@ def _keyEcho(event):
                 # until this can be fixed.]]]
                 #
                 #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    event_string += " Off"
+                #    event_string += _(" off")
                 #else:
-                #    event_string += " On"
+                #    event_string += _(" on")
                 pass
 
         elif _isFunctionKey(event_string):
@@ -704,13 +619,16 @@ def _keyEcho(event):
         if event_string in keynames.keynames:
             event_string = keynames.keynames[event_string]
 
+        if lockState:
+            event_string += lockState
+
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
 
         # We keep track of the time as means to let others know that
         # we are probably echoing a key and should not be interrupted.
         #
-        lastKeyEchoTime = time.time()
+        orca_state.lastKeyEchoTime = time.time()
 
         speech.speak(event_string, voice)
 
@@ -727,10 +645,10 @@ def _processKeyboardEvent(event):
 
     Returns True if the event should be consumed.
     """
-
-    global lastInputEvent
     global _orcaModifierPressed
 
+    orca_state.lastInputEvent = event.timestamp
+
     # Log the keyboard event for future playback, if desired.
     # Note here that the key event_string being output is
     # exactly what we received.  The KeyboardEvent object,
@@ -739,7 +657,7 @@ def _processKeyboardEvent(event):
     #
     string = atspi.KeystrokeListener.keyEventToString(event)
     if _recordingKeystrokes and _keystrokesFile \
-       and (event_string != "Pause"):
+       and (event.event_string != "Pause"):
         _keystrokesFile.write(string + "\n")
     debug.printInputEvent(debug.LEVEL_FINE, string)
 
@@ -771,7 +689,7 @@ def _processKeyboardEvent(event):
         _orcaModifierPressed = False
 
     if _orcaModifierPressed:
-        keyboardEvent.modifiers |= (1 << MODIFIER_ORCA)
+        keyboardEvent.modifiers |= (1 << settings.MODIFIER_ORCA)
 
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
@@ -795,15 +713,11 @@ def _processKeyboardEvent(event):
                 if event_string in keynames.keynames:
                     event_string = keynames.keynames[event_string]
                 speech.speak(event_string)
-            elif (keyboardEvent.type \
-                  == atspi.Accessibility.KEY_RELEASED_EVENT) \
-                 and (keyboardEvent.event_string == "Escape"):
-                exitLearnMode(None, keyboardEvent)
             consumed = True
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-    lastInputEvent = keyboardEvent
+    orca_state.lastInputEvent = keyboardEvent
 
     return consumed or isOrcaModifier
 
@@ -822,8 +736,6 @@ def _processBrailleEvent(command):
     Returns True if the event was consumed; otherwise False
     """
 
-    global lastInputEvent
-
     # [[[TODO: WDW - probably should add braille bindings to this module.]]]
 
     consumed = False
@@ -833,7 +745,7 @@ def _processBrailleEvent(command):
     speech.stop()
 
     event = BrailleEvent(command)
-    lastInputEvent = event
+    orca_state.lastInputEvent = event
 
     try:
         consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
@@ -846,84 +758,6 @@ def _processBrailleEvent(command):
 
     return consumed
 
-########################################################################
-#                                                                      #
-# METHODS FOR DRAWING RECTANGLES AROUND OBJECTS ON THE SCREEN          #
-#                                                                      #
-########################################################################
-
-_display = None
-_visibleRectangle = None
-
-def drawOutline(x, y, width, height, erasePrevious=True):
-    """Draws a rectangular outline around the accessible, erasing the
-    last drawn rectangle in the process."""
-
-    global _display
-    global _visibleRectangle
-
-    if not _display:
-        try:
-            _display = gtk.gdk.display_get_default()
-        except:
-            debug.printException(debug.LEVEL_FINEST)
-            _display = gtk.gdk.display(":0")
-
-        if not _display:
-            debug.println(debug.LEVEL_SEVERE,
-                          "orca.drawOutline could not open display.")
-            return
-
-    screen = _display.get_default_screen()
-    root_window = screen.get_root_window()
-    graphics_context = root_window.new_gc()
-    graphics_context.set_subwindow(gtk.gdk.INCLUDE_INFERIORS)
-    graphics_context.set_function(gtk.gdk.INVERT)
-    graphics_context.set_line_attributes(3,                  # width
-                                         gtk.gdk.LINE_SOLID, # style
-                                         gtk.gdk.CAP_BUTT,   # end style
-                                         gtk.gdk.JOIN_MITER) # join style
-
-    # Erase the old rectangle.
-    #
-    if _visibleRectangle and erasePrevious:
-        drawOutline(_visibleRectangle[0], _visibleRectangle[1],
-                    _visibleRectangle[2], _visibleRectangle[3], False)
-        _visibleRectangle = None
-
-    # We'll use an invalid x value to indicate nothing should be
-    # drawn.
-    #
-    if x < 0:
-        _visibleRectangle = None
-        return
-
-    # The +1 and -2 stuff here is an attempt to stay within the
-    # bounding box of the object.
-    #
-    root_window.draw_rectangle(graphics_context,
-                               False, # Fill
-                               x + 1,
-                               y + 1,
-                               max(1, width - 2),
-                               max(1, height - 2))
-
-    _visibleRectangle = [x, y, width, height]
-
-def outlineAccessible(accessible, erasePrevious=True):
-    """Draws a rectangular outline around the accessible, erasing the
-    last drawn rectangle in the process."""
-
-    if accessible:
-        component = accessible.component
-        if component:
-            visibleRectangle = component.getExtents(0) # coord type = screen
-            drawOutline(visibleRectangle.x, visibleRectangle.y,
-                        visibleRectangle.width, visibleRectangle.height,
-                        erasePrevious)
-    else:
-        drawOutline(-1, 0, 0, 0, erasePrevious)
-
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING INITIALIZATION, SHUTDOWN, AND USE.              #
@@ -954,6 +788,8 @@ def loadUserSettings(script=None, inputEvent=None):
     global _userSettings
 
     # Shutdown the output drivers and give them a chance to die.
+    #
+    httpserver.shutdown()
     speech.shutdown()
     braille.shutdown()
     mag.shutdown()
@@ -1008,6 +844,8 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
+    httpserver.init()
+
     return True
 
 def _showPreferencesGUI(script=None, inputEvent=None):
@@ -1067,139 +905,35 @@ def init(registry):
     if _initialized:
         return False
 
+    # Do not hang on initialization if we can help it.
+    #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
+    # Note that we have moved the Orca specific keybindings to the default
+    # script, so _keyBindings is currently empty. The logic is retained
+    # here, just in case we wish to reinstate them in the future.
+    #
     _keyBindings = keybindings.KeyBindings()
 
-    enterLearnModeHandler = InputEventHandler(\
-        enterLearnMode,
-        _("Enters learn mode.  Press escape to exit learn mode."))
-    _keyBindings.add(keybindings.KeyBinding("F1", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA, \
-                                            enterLearnModeHandler))
-
-    decreaseSpeechRateHandler = InputEventHandler(\
-        speech.decreaseSpeechRate,
-        _("Decreases the speech rate."))
-    _keyBindings.add(keybindings.KeyBinding("Left", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            decreaseSpeechRateHandler))
-
-    increaseSpeechRateHandler = InputEventHandler(\
-        speech.increaseSpeechRate,
-        _("Increases the speech rate."))
-    _keyBindings.add(keybindings.KeyBinding("Right", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            increaseSpeechRateHandler))
-
-    shutdownHandler = InputEventHandler(shutdown, _("Quits Orca"))
-    _keyBindings.add(keybindings.KeyBinding("F12", \
-                                            0, \
-                                            0,
-                                            shutdownHandler))
-    _keyBindings.add(keybindings.KeyBinding("SunF37", \
-                                            0, \
-                                            0,
-                                            shutdownHandler))
-
-    keystrokeRecordingHandler = InputEventHandler(\
-        toggleKeystrokeRecording,
-        _("Toggles keystroke recording on and off."))
-    _keyBindings.add(keybindings.KeyBinding("Pause", \
-                                            0, \
-                                            0,
-                                            keystrokeRecordingHandler))
-
-    preferencesSettingsHandler = InputEventHandler(\
-        _showPreferencesGUI,
-        _("Displays the preferences configuration dialog."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "space", \
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        preferencesSettingsHandler))
-
-    loadUserSettingsHandler = InputEventHandler(\
-        loadUserSettings,
-        _("Reloads user settings and reinitializes services as necessary."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "space", \
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        loadUserSettingsHandler))
-
-    toggleSilenceSpeechHandler = InputEventHandler(\
-        _toggleSilenceSpeech,
-        _("Toggles the silencing of speech."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "s", \
-        1 << MODIFIER_ORCA,
-        1 << MODIFIER_ORCA,
-        toggleSilenceSpeechHandler))
-
-    listAppsHandler = InputEventHandler(
-        printApps,
-        _("Prints a debug listing of all known applications to the console where Orca is running."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "F5",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        listAppsHandler))
-
-    cycleDebugLevelHandler = InputEventHandler(
-        cycleDebugLevel,
-        _("Cycles the debug level at run time."))
-    _keyBindings.add(keybindings.KeyBinding(
-        "F5",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        cycleDebugLevelHandler))
-
-    printActiveAppHandler = InputEventHandler(\
-        printActiveApp,
-        _("Prints debug information about the currently active application to the console where Orca is running."))
-    _keyBindings.add(keybindings.KeyBinding("F6", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            printActiveAppHandler))
-
-    printAncestryHandler = InputEventHandler(\
-        printAncestry,
-        _("Prints debug information about the ancestry of the object with focus"))
-    _keyBindings.add(keybindings.KeyBinding(
-        "F7",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        1 << MODIFIER_ORCA,
-        printAncestryHandler))
-
-    printHierarchyHandler = InputEventHandler(\
-        printHierarchy,
-        _("Prints debug information about the application with focus"))
-
-    _keyBindings.add(keybindings.KeyBinding(
-        "F7",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
-        printHierarchyHandler))
-
-    nextPresentationManagerHandler = InputEventHandler(\
-        _switchToNextPresentationManager,
-        _("Switches to the next presentation manager."))
-    _keyBindings.add(keybindings.KeyBinding("F8", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA,
-                                            nextPresentationManagerHandler))
-
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
                                    "object:children-changed:")
 
+    # We also want to stop speech when a mouse button is pressed.
+    #
+    registry.registerEventListener(_onMouseButton,
+                                   "mouse:button")
+
     loadUserSettings()
 
     registry.registerKeystrokeListeners(_processKeyboardEvent)
 
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
     _initialized = True
     return True
 
@@ -1212,6 +946,12 @@ def start(registry):
     if not _initialized:
         init(registry)
 
+    # Do not hang on startup if we can help it.
+    #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
     try:
         speech.speak(_("Welcome to Orca."))
         braille.displayMessage(_("Welcome to Orca."))
@@ -1223,7 +963,7 @@ def start(registry):
         # [[[WDW - comment out hierarchical_presenter for now.  It
         # relies on the list of known applications, and we've disabled
         # that due to a hang in a call to getChildAtIndex in
-        # atspi.getKnownApplications.]]]
+        # util.getKnownApplications.]]]
         #
         #import focus_tracking_presenter
         #import hierarchical_presenter
@@ -1237,14 +977,19 @@ def start(registry):
 
     _switchToPresentationManager(0) # focus_tracking_presenter
 
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
     registry.start()
 
-def abort():
-    os._exit(1)
+def abort(exitCode=1):
+    os._exit(exitCode)
 
-def timeout():
-    print "TIMEOUT: Hung while trying to shutdown.  Aborting."
-    abort()
+def timeout(signum=None, frame=None):
+    debug.println(debug.LEVEL_SEVERE,
+                  "TIMEOUT: something has hung.  Aborting.")
+    debug.printStack(debug.LEVEL_ALL)
+    abort(50)
 
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
@@ -1259,12 +1004,11 @@ def shutdown(script=None, inputEvent=None):
     if not _initialized:
         return False
 
-    # [[[TODO: WDW - the timer stuff is an experiment to see if
-    # we can recover from hangs.  It's only experimental, so it's
-    # commented out for now.]]]
+    # Try to say goodbye, but be defensive if something has hung.
     #
-    #timer = threading.Timer(5.0, timeout)
-    #timer.start()
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
 
     speech.speak(_("goodbye."))
     braille.displayMessage(_("Goodbye."))
@@ -1274,6 +1018,8 @@ def shutdown(script=None, inputEvent=None):
     registry = atspi.Registry()
     registry.deregisterEventListener(_onChildrenChanged,
                                      "object:children-changed:")
+    registry.deregisterEventListener(_onMouseButton,
+                                     "mouse:button")
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].deactivate()
@@ -1289,8 +1035,8 @@ def shutdown(script=None, inputEvent=None):
 
     registry.stop()
 
-    #timer.cancel()
-    #del timer
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
 
     _initialized = False
     return True
@@ -1299,35 +1045,49 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    #print "Shutting down and exiting due to signal =", signum
+    debug.println(debug.LEVEL_ALL,
+                  "Shutting down and exiting due to signal = %d" \
+                  % signum)
+
+    debug.println(debug.LEVEL_ALL, "Current stack is:")
+    debug.printStack(debug.LEVEL_ALL)
 
     # Well...we'll try to exit nicely, but if we keep getting called,
     # something bad is happening, so just quit.
     #
     if exitCount:
-        abort()
+        abort(signum)
     else:
         exitCount += 1
 
     # Try to do a graceful shutdown if we can.
     #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
     try:
         if _initialized:
             shutdown()
-            return
         else:
             # We always want to try to shutdown speech since the
             # speech servers are very persistent about living.
             #
             speech.shutdown()
             shutdown()
-            os._exit(0)
     except:
-        abort()
+        pass
+
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
+    abort(signum)
 
 def abortOnSignal(signum, frame):
-    #print "Aborting due to signal =", signum
-    abort()
+    debug.println(debug.LEVEL_ALL,
+                  "Aborting due to signal = %d" \
+                  % signum)
+    abort(signum)
 
 def usage():
     """Prints out usage information."""
@@ -1345,6 +1105,18 @@ def usage():
     pass
 
 def main():
+    """The main entry point for Orca.  The exit codes for Orca will
+    loosely be based on signals, where the exit code will be the
+    signal used to terminate Orca (if a signal was used).  Otherwise,
+    an exit code of 0 means normal completion and an exit code of 50
+    means Orca exited because of a hang."""
+
+    # Method to call when we think something might be hung.
+    #
+    settings.timeoutCallback = timeout
+
+    # Various signal handlers we want to listen for.
+    #
     signal.signal(signal.SIGHUP, shutdownOnSignal)
     signal.signal(signal.SIGINT, shutdownOnSignal)
     signal.signal(signal.SIGTERM, shutdownOnSignal)
@@ -1361,15 +1133,6 @@ def main():
     except:
         pass
 
-    # Do not run Orca if accessibility has not been enabled.
-    #
-    import commands
-    a11yEnabled = commands.getoutput(\
-        "gconftool-2 --get /desktop/gnome/interface/accessibility")
-    if a11yEnabled != "true":
-        _showPreferencesConsole()
-        abort()
-
     # Parse the command line options.
     #
     # Run the preferences setup if the user has specified
@@ -1381,41 +1144,50 @@ def main():
     setupRequested  = False
     showGUI         = False
     try:
-        # Note to translators: ? is for help
-        #                      h is for help
-        #                      s is for setup
-        #                      n is for no setup
-        #                      t is for text setup
-        #                      v is for version
+        # ? is for help
+        # h is for help
+        # s is for setup
+        # n is for no setup
+        # t is for text setup
+        # v is for version
         #
         opts, args = getopt.getopt(
             sys.argv[1:],
-            _("?stnv"),
-            [_("help"),
-             _("setup"),
-             _("gui-setup"),
-             _("text-setup"),
-             _("no-setup"),
-             _("version")])
+            "?stnv",
+            ["help",
+             "setup",
+             "gui-setup",
+             "text-setup",
+             "no-setup",
+             "version"])
         for opt, val in opts:
-            if opt in (_("-s"), _("--gui-setup"), _("--setup")):
+            if opt in ("-s", "--gui-setup", "--setup"):
                 setupRequested = True
                 showGUI = desktopRunning
-            if opt in (_("-t"), _("--text-setup")):
+            if opt in ("-t", "--text-setup"):
                 setupRequested = True
                 showGUI = False
-            if opt in (_("-n"), _("--no-setup")):
+            if opt in ("-n", "--no-setup"):
                 bypassSetup = True
-            if opt in (_("-?"), _("--help")):
+            if opt in ("-?", "--help"):
                 usage()
                 os._exit(0)
-            if opt in (_("-v"), _("--version")):
+            if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
                 os._exit(0)
     except:
         usage()
         os._exit(2)
 
+    # Do not run Orca if accessibility has not been enabled.
+    #
+    import commands
+    a11yEnabled = commands.getoutput(\
+        "gconftool-2 --get /desktop/gnome/interface/accessibility")
+    if a11yEnabled != "true":
+        _showPreferencesConsole()
+        abort()
+
     if setupRequested and (not bypassSetup) and (not showGUI):
         _showPreferencesConsole()
 
