@@ -20,8 +20,11 @@
 """Uses speech prompts and a command line interface to set Orca
 user preferences."""
 
-import sys
-import time
+__id__        = "$Id: orca_console_prefs.py,v 1.9 2006/06/10 00:54:47 wwalker Exp $"
+__version__   = "$Revision: 1.9 $"
+__date__      = "$Date: 2006/06/10 00:54:47 $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import acss
 import settings
