@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py,v 1.62 2006/06/22 18:23:44 richb Exp $"
-__version__   = "$Revision: 1.62 $"
-__date__      = "$Date: 2006/06/22 18:23:44 $"
+__id__        = "$Id: settings.py,v 1.72 2006/07/28 19:49:30 wwalker Exp $"
+__version__   = "$Revision: 1.72 $"
+__date__      = "$Date: 2006/07/28 19:49:30 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -106,6 +106,11 @@ consolePreferencesModule= "orca_console_prefs"
 #
 orcaModifierKeys        = ["Insert", "KP_Insert"]
 
+# A new modifier to use (set by the press of any key in the
+# orcaModifierKeys list) to represent the Orca modifier.
+#
+MODIFIER_ORCA = 8
+
 # Verbosity levels (see setBrailleVerbosityLevel and
 # setSpeechVerbosityLevel).  These will have an impact on the various
 # individual verbosity levels for rolenames, accelerators, etc.
@@ -319,6 +324,36 @@ cacheAccessibles        = True
 #
 learnModeEnabled        = False
 
+# If non-zero, we use time.sleep() in various places to attempt to
+# free up the global interpreter lock.  Take a look at the following
+# URLs for more information:
+#
+# http://mail.python.org/pipermail/python-list/2002-October/126632.html
+# http://twistedmatrix.com/pipermail/twisted-python/2005-July/011052.html
+# http://www.pyzine.com/Issue001/Section_Articles/article_ThreadingGlobalInterpreter.html
+#
+gilSleepTime            = 0.00001
+
+# If True, we use the bonobo main loop provided by bonobo to handle
+# all events in atspi.py.  If False, we create our own loop.
+#
+useBonoboMain           = True
+
+# If True, we output debug information for the event queue.  We
+# use this in addition to log level to prevent debug logic from
+# bogging down event handling.
+#
+debugEventQueue         = False
+
+# The timeout value (in seconds) and callback used to determine if
+# Orca has hung or not.  The only setting one should muck with here is
+# the timeoutTime unless you want to create a custom callback handler
+# for the timeout.  See braille.py, atspi.py, and orca.py:init for how
+# these are used.
+#
+timeoutTime             = 10   # a value of 0 means don't do hang checking
+timeoutCallback         = None # Set by orca.py:init to orca.timeout
+
 # Assists with dealing with CORBA COMM_FAILURES.  A failure doesn't
 # always mean an object disappeared - there just might be a network
 # glitch.  So, on COMM_FAILURES, we might retry a few times before
@@ -327,6 +362,13 @@ learnModeEnabled        = False
 commFailureWaitTime     = 0.1
 commFailureAttemptLimit = 5
 
+# Keyboard double-click period. If the same key is pressed within
+# this time period, it's considered to be a double-click and might
+# provide different functionality (for example, Numpad 5 double-click
+# spells the current word rather than speaks it).
+#
+doubleClickTimeout = 0.5
+
 # Which packages to search, and the order in which to search,
 # for custom scripts.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
