@@ -20,16 +20,16 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py,v 1.57 2006/07/21 19:49:03 wwalker Exp $"
-__version__   = "$Revision: 1.57 $"
-__date__      = "$Date: 2006/07/21 19:49:03 $"
+__id__        = "$Id: speech.py,v 1.58 2006/07/28 17:33:17 wwalker Exp $"
+__version__   = "$Revision: 1.58 $"
+__date__      = "$Date: 2006/07/28 17:33:17 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import time
 
 import debug
-import orca
+import orca_state
 import platform
 import settings
 
@@ -137,9 +137,9 @@ def speak(text, acss=None, interrupt=True):
 
     # We will not interrupt a key echo in progress.
     #
-    if orca.lastKeyEchoTime:
+    if orca_state.lastKeyEchoTime:
         interrupt = interrupt \
-            and ((time.time() - orca.lastKeyEchoTime) > 0.5)
+            and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
 
     if settings.silenceSpeech:
         return
@@ -169,9 +169,9 @@ def speakUtterances(utterances, acss=None, interrupt=True):
 
     # We will not interrupt a key echo in progress.
     #
-    if orca.lastKeyEchoTime:
+    if orca_state.lastKeyEchoTime:
         interrupt = interrupt \
-            and ((time.time() - orca.lastKeyEchoTime) > 0.5)
+            and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
 
     if settings.silenceSpeech:
         return
